@@ -1,5 +1,5 @@
 /*
- * S.F.S. v. 0.5alpha-linux
+ * T.C.F.S. v. 1.0beta-linux
  *
  * sfsd		This program handles RPC "NFS" data requests 
  *              adopting a secure transfer protocol.
@@ -70,6 +70,7 @@ extern unsigned long sec,msec;
 extern void receivekey (int);
 
 #define desall(x)  ((unsigned long)(((unsigned long)x) & 0xfffffff8L))
+#define inpure(x)  ((unsigned long)(((unsigned long)x) & 0x7L))
 mtable *first;
 static char iobuf[NFS_MAXDATA];
 static char pathbuf[NFS_MAXPATHLEN + 1];
@@ -402,7 +403,7 @@ readargs *argp;
 #ifdef BOF
   syslog(LOG_DAEMON,"read_2");
 #endif
-  kflag=(h->extended==7)?1:0;
+  kflag=IS_CRYPTED(h->extended);
   /* Test file owner */
   if (kflag && uid!=h->uid) {
     syslog(LOG_DAEMON,"Fuck off intruder !!!!");
@@ -443,7 +444,7 @@ readargs *argp;
     h->offset= newof;
     /* we have a buffer with intersection */
     if (kflag) { /* Decrypt the buffer */
-      if(sfs_decrypt(h->buffer,h->valid,h->offset,uid)){
+      if(sfs_decrypt(h->buffer,desall(h->valid),h->offset,uid)){
 	errno=22;
 	result.readres.status=NFSERR_ACCES;
 	return nfs_errno();
@@ -482,7 +483,7 @@ readargs *argp;
     h->offset= newof;
     /* We have a buffer with intersaction with the block requested */
     if (kflag) { /* Decrypt the buffer */
-      if(sfs_decrypt(h->buffer,h->valid,h->offset,uid)) {
+      if(sfs_decrypt(h->buffer,desall(h->valid),h->offset,uid)) {
 	errno=22;
 	result.readres.status=NFSERR_ACCES;
 	return nfs_errno();
@@ -539,7 +540,7 @@ readargs *argp;
 	  iobuf[j+argp->count]=h->buffer[j];
       }
       if (kflag) { 
-	if(sfs_decrypt(h->buffer,h->valid,h->offset,uid)){
+	if(sfs_decrypt(h->buffer,desall(h->valid),h->offset,uid)){
 	  errno=22;
 	  result.readres.status=NFSERR_ACCES;
 	  return nfs_errno();
@@ -592,7 +593,7 @@ writeargs *argp;
   if ((h=fh_find((svc_fh *)&argp->file,0))==NULL) {
     return NFSERR_STALE;
   }
-  kflag=(h->extended==7)?1:0;
+  kflag=IS_CRYPTED(h->extended);
  if (h->valid == -1) {
     syslog(LOG_DAEMON,"aiutoooooooooooooo");
     return NFSERR_IO;
@@ -616,19 +617,45 @@ writeargs *argp;
 	h->offset=argp->offset;
       h->valid=read(fd,h->buffer,8192); /* load the buffer */
       if (kflag) { /* decrypt the buffer */
-	if (sfs_decrypt(h->buffer,h->valid,h->offset,uid))
+	if (sfs_decrypt(h->buffer,desall(h->valid),h->offset,uid))
 	  return NFSERR_PERM;
       }
     } else { /* i'm appending at the end of the file */
       memset(h->buffer,'\0',8192);
+      errno=0;
+      if (kflag) {
       /* qua bisogna aggiungere la gestione del vecchio fine file
 	 perche' se il buffer contiene alcuni byte spuri crittografati in qualche
 	 modo, dobbiamo fare in modo che tutto ritorni a posto... */
-      if (kflag)
-	h->offset=desall(argp->offset);
-      else
+
+	(void) lseek(fd, desall(bstat.st_size), L_SET);
+	if (errno) {
+	  fd_inactive(fd);
+	  return (nfs_errno());
+	}	
+	h->valid=read(fd,h->buffer,inpure(bstat.st_size)); /*load the buffer */
+	h->offset=desall(bstat.st_size);
+	if (argp->offset>h->offset+inpure(bstat.st_size)+1) { /* I'm not appending exactly at the end of the file */
+	  if (sfs_crypt(h->buffer,8,h->offset,uid))
+	    return NFSERR_PERM;
+	  errno=0;
+	  (void) lseek(fd, desall(bstat.st_size), L_SET);
+	  if (errno) {
+	    fd_inactive(fd);
+	    return (nfs_errno());
+	  }	
+	  if (write(fd,h->buffer,8)!=8)
+	    return NFSERR_IO;
+	  memset(h->buffer,'\0',8192);
+	  h->offset=desall(argp->offset);
+	  h->valid=argp->data.data_len+(argp->offset - h->offset)-1;	
+	} else {
+	  h->valid=argp->data.data_len+h->valid;
+	}
+      } else {
 	h->offset=argp->offset;
-      h->valid=argp->data.data_len+(argp->offset - h->offset)-1;
+	h->valid=argp->data.data_len+(argp->offset - h->offset)-1;      
+      }
     }
   } else {
     if (!(h->offset<=argp->offset && argp->offset-h->offset<h->valid)) {
@@ -650,19 +677,46 @@ writeargs *argp;
 	  h->offset=argp->offset;
 	h->valid=read(fd,h->buffer,8192); /* load the buffer */
 	if (kflag) { /* decrypt the block */
-	  if (sfs_decrypt(h->buffer,h->valid,h->offset,uid))
+	  if (sfs_decrypt(h->buffer,desall(h->valid),h->offset,uid))
 	    return NFSERR_PERM;
 	}
       } else { /* the block is at the end of the file */
-	memset(h->buffer,'\0',8192);
+/*	memset(h->buffer,'\0',8192); */
+	if (kflag) {
+	  /* qua bisogna aggiungere la gestione del vecchio fine file
+	     perche' se il buffer contiene alcuni byte spuri crittografati in qualche
+	     modo, dobbiamo fare in modo che tutto ritorni a posto... */
+	  if (argp->offset -h->offset > h->valid+1) { /* I'm not appending exactly at the end of the file */
+	    memset(h->buffer+h->valid+1,'\0',8192-h->valid-1);
+	    if (sfs_crypt(h->buffer,desall(h->valid),h->offset,uid))
+	      return NFSERR_PERM;
+	    errno=0;
+	    (void) lseek(fd, h->offset, L_SET);
+	    if (errno) {
+	      fd_inactive(fd);
+	      return (nfs_errno());
+	    }	
+	    if (write(fd,h->buffer,h->valid+8-inpure(h->valid))!=h->valid+8-inpure(h->valid))
+	      return NFSERR_IO;
+	    memset(h->buffer,'\0',8192);
+	    h->offset=desall(argp->offset);
+	    h->valid=argp->data.data_len+(argp->offset - h->offset)-1;	
+	  } else {
+	    /*h->offset=desall(argp->offset);
+	    h->valid=argp->data.data_len+(argp->offset - h->offset)-1-h->valid;*/
+	  }
+	} else {
+	  h->offset=argp->offset;
+	  h->valid=argp->data.data_len+(argp->offset - h->offset)-1;      
+	}
 	/* qua bisogna aggiungere la gestione del vecchio fine file
 	   perche' se il buffer contiene alcuni byte spuri crittografati in qualche
 	   modo, dobbiamo fare in modo che tutto ritorni a posto... */
-	if (kflag)
+/*	if (kflag)
 	  h->offset=desall(argp->offset);
 	else
 	  h->offset=argp->offset;
-	h->valid=argp->data.data_len+(argp->offset - h->offset)-1;
+	h->valid=argp->data.data_len+(argp->offset - h->offset)-1; */
       } 
     }
   }
@@ -680,14 +734,14 @@ writeargs *argp;
     j=argp->offset-tmpoff; /* bytes copied into iobuf */
     bcopy(argp->data.data_val,iobuf+j,argp->data.data_len);
     j+=argp->data.data_len;
-    if ( kflag && (j%8) != 0) {
+    if ( kflag && ((j%8) != 0) && j+tmpoff-h->offset<h->valid && j+8-j&8+tmpoff-h->offset < h->valid) {
       /* i have to align to des block */
       bcopy(&h->buffer[j+tmpoff-h->offset],iobuf+j,8-j%8);
       j+=8-j%8;
     }
     bcopy(iobuf,&h->buffer[tmpoff-h->offset],j); /* update cache */
     if (kflag) { 
-	if (sfs_crypt(iobuf,j,tmpoff,uid))
+	if (sfs_crypt(iobuf,desall(j),tmpoff,uid))
 	  return NFSERR_PERM;
       }
     errno=0;
@@ -714,11 +768,11 @@ writeargs *argp;
 	bcopy(&h->buffer[tmpoff-h->offset],iobuf,8);
       }
       j=argp->offset-tmpoff; /* bytes copied in iobuf */
-      bcopy(argp->data.data_val,&iobuf[argp->offset-tmpoff],8192-(argp->offset-h->offset));
+      bcopy(argp->data.data_val,&iobuf[j],8192-(argp->offset-h->offset));
       j+=8192-(argp->offset-h->offset);
       bcopy(iobuf,&h->buffer[tmpoff],j); /* update the cache */
       if (kflag) { 
-	if (sfs_crypt(iobuf,j,tmpoff,uid))
+	if (sfs_crypt(iobuf,desall(j),tmpoff,uid))
 	  return NFSERR_PERM;
       }
       errno=0;
@@ -735,7 +789,7 @@ writeargs *argp;
       }
       k=read(fd,iobuf,8192);
       if (kflag) { 
-	if (sfs_decrypt(iobuf,k,h->offset+8192,uid))
+	if (sfs_decrypt(iobuf,desall(k),h->offset+8192,uid))
 	  return NFSERR_PERM;
       }
       switch (k) {
@@ -746,7 +800,7 @@ writeargs *argp;
       case 0: /* i have to append to the file */
 	bcopy(argp->data.data_val+8192-(argp->offset-h->offset),iobuf,argp->data.data_len-(8192-(argp->offset-h->offset))); 
 	if (kflag) { 
-	  if (sfs_crypt(iobuf,argp->data.data_len-(8192-(argp->offset-h->offset)),h->offset+8192,uid))
+	  if (sfs_crypt(iobuf,desall(argp->data.data_len-(8192-(argp->offset-h->offset))),h->offset+8192,uid))
 	    return NFSERR_PERM;
 	}
 	write(fd,iobuf,argp->data.data_len-(8192-(argp->offset-h->offset)));
@@ -760,7 +814,7 @@ writeargs *argp;
 	j=argp->data.data_len-(8192-(argp->offset-h->offset)); /* copy remaining bytes */
 	k=k>j?k:j; /* bytes to write */
 	if (kflag) { 
-	  if (sfs_crypt(iobuf,k,h->offset+8192,uid))
+	  if (sfs_crypt(iobuf,desall(k),h->offset+8192,uid))
 	    return NFSERR_PERM;
 	}
 	(void) lseek (fd,(long)h->offset+8192,L_SET);
@@ -788,12 +842,14 @@ writeargs *argp;
 	k=desall(h->valid)+8L;
       else
 	k=h->valid;
-      bcopy(argp->data.data_val,&iobuf[j],k-(argp->offset-h->offset));
-      j+=(k-(argp->offset-h->offset))>0?(k-(argp->offset-h->offset)):0;
+      bcopy(argp->data.data_val,&iobuf[j],argp->data.data_len);
+      j+=argp->data.data_len;
+/*      bcopy(argp->data.data_val,&iobuf[j],k-(argp->offset-h->offset));
+      j+=(k-(argp->offset-h->offset))>0?(k-(argp->offset-h->offset)):0;*/
       h->valid=0; /* invalid the cache so the block will be reread */
       errno=0;
       if (kflag) { 
-	if (sfs_crypt(iobuf,j,tmpoff,uid))
+	if (sfs_crypt(iobuf,desall(j),tmpoff,uid))
 	  return NFSERR_PERM;
       }
       (void) lseek (fd,(long) tmpoff,L_SET);
@@ -804,15 +860,15 @@ writeargs *argp;
       if (write(fd,iobuf,j)!= j) {
 	dprintf(1,"Write failure, errno is %d.\n",errno);
       }
-      if (argp->data.data_len>j) {
+/*      if (argp->data.data_len>j) {
 	bcopy(argp->data.data_val+j,iobuf,argp->data.data_len-j);
-	/* qui bisogna inserire le condizioni di fine file */
+	/* qui bisogna inserire le condizioni di fine file 
 	if (kflag) { 
-	  if (sfs_crypt(iobuf,argp->data.data_len-j,tmpoff+j,uid))
+	  if (sfs_crypt(iobuf,desall(argp->data.data_len-j),tmpoff+j,uid))
 	    return NFSERR_PERM;
 	}      
 	write(fd,iobuf,argp->data.data_len-j);
-      }
+      } */
       fd_inactive(fd);
       if (errno)
 	return nfs_errno();
